@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' package for R.
- * Copyright (c) 2013-2017, Marek Gagolewski and other contributors.
+ * Copyright (c) 2013-2019, Marek Gagolewski and other contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,6 +59,13 @@ StriContainerRegexPattern::StriContainerRegexPattern(SEXP rstr, R_len_t _nrecycl
    this->lastMatcherIndex = -1;
    this->lastMatcher = NULL;
    this->flags = _flags;
+
+   R_len_t n = get_n();
+   for (R_len_t i=0; i<n; ++i) {
+      if (!isNA(i) && get(i).length() <= 0) {
+         Rf_warning(MSG__EMPTY_SEARCH_PATTERN_UNSUPPORTED);
+      }
+   }
 }
 
 
@@ -140,6 +147,9 @@ RegexMatcher* StriContainerRegexPattern::getMatcher(R_len_t i)
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Disallow NA options
+ *
+ * @version 1.1.6 (Marek Gagolewski, 2017-11-10)
+ *    PROTECT STRING_ELT(names, i)
  */
 uint32_t StriContainerRegexPattern::getRegexFlags(SEXP opts_regex)
 {
@@ -159,8 +169,7 @@ uint32_t StriContainerRegexPattern::getRegexFlags(SEXP opts_regex)
          if (STRING_ELT(names, i) == NA_STRING)
             Rf_error(MSG__REGEXP_CONFIG_FAILED); // error() call allowed here
 
-         const char* curname = CHAR(STRING_ELT(names, i));
-
+         const char* curname = stri__copy_string_Ralloc(STRING_ELT(names, i), "curname");  /* this is R_alloc'ed */
          if  (!strcmp(curname, "case_insensitive")) {
             bool val = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_regex, i), "case_insensitive");
             if (val) flags |= UREGEX_CASE_INSENSITIVE;
